@@ -26,6 +26,7 @@
 import logging
 logger = logging.getLogger(__name__)
 
+import os
 import pathlib
 import json
 import configparser
@@ -43,6 +44,10 @@ class Config(configparser.ConfigParser, object):  # python 2 fix
     """
     #: `dict`-tree of presenter layouts for various modes
     layout = {}
+    #: `dict`-tree of presenter layouts that are not configurable
+    static_layout = {
+        'deck-overview': 'deck',
+    }
 
     #: `dict` of strings that are the valid representations of widgets from the presenter window
     #: that can be dynamically rearranged, mapping to their names
@@ -54,7 +59,8 @@ class Config(configparser.ConfigParser, object):  # python 2 fix
         'notes':      (set(placeable_widgets.keys()) - {"annotations", "highlight"}, {"annotations"}),
         'plain':      (set(placeable_widgets.keys()) - {"notes", "highlight"},),
         'note_pages': (set(placeable_widgets.keys()) - {"current", "highlight"},),
-        'highlight':  ({"highlight"}, set(placeable_widgets.keys()) - {"highlight"})
+        'highlight':  ({"highlight"}, set(placeable_widgets.keys()) - {"highlight"}),
+        'highlight_notes':  ({"highlight"}, set(placeable_widgets.keys()) - {"highlight"}),
     }
 
     #: `dict` mapping accelerator keys to actions
@@ -126,6 +132,11 @@ class Config(configparser.ConfigParser, object):  # python 2 fix
         config.read(util.get_default_config())
         config.load_window_layouts()
 
+        # modify defaults
+        if 'I3SOCK' in os.environ:
+            logger.info('Detected i3 window manager, disabled content window starting fullscreen by default')
+            config.set('content', 'start_fullscreen', 'off')
+
         all_commands = dict(config.items('shortcuts')).keys()
 
         config.read(config.path_to_config(True))
@@ -488,7 +499,10 @@ class Config(configparser.ConfigParser, object):  # python 2 fix
     def get_layout(self, layout_name):
         """ Getter for the `~layout_name` layout.
         """
-        return self.layout[layout_name]
+        try:
+            return self.layout[layout_name]
+        except KeyError:
+            return self.static_layout[layout_name]
 
 
     def update_layout_from_widgets(self, layout_name, widget, pane_handle_pos):
@@ -499,4 +513,6 @@ class Config(configparser.ConfigParser, object):  # python 2 fix
             widget (:class:`~Gtk.Widget`): the widget that will contain the layout.
             pane_handle_pos (`dict`): Map of :class:`~Gtk.Paned` to the relative handle position (float in 0..1)
         """
+        if layout_name not in self.static_layout:
+            return
         self.update_layout_tree(layout_name, self.widget_layout_to_tree(widget, pane_handle_pos))
