@@ -92,7 +92,7 @@ class Scribbler(builder.Builder):
     scribble_clear = None
 
     #: A :class:`~Gtk.OffscreenWindow` where we render the scribbling interface when it's not shown
-    off_render = None
+    scribble_off_render = None
     #: :class:`~Gtk.Box` in the Presenter window, where we insert scribbling.
     p_central = None
 
@@ -163,7 +163,7 @@ class Scribbler(builder.Builder):
 
         self.load_ui('highlight')
         builder.load_widgets(self)
-        self.get_application().add_window(self.off_render)
+        self.get_application().add_window(self.scribble_off_render)
 
         self.on_draw = builder.get_callback_handler('on_draw')
         self.track_motions = builder.get_callback_handler('track_motions')
@@ -404,19 +404,31 @@ class Scribbler(builder.Builder):
         window = self.c_da.get_window()
 
         if window is None:
-            return ValueError('Cannot initialize scribble acche without drawing area window')
+            return ValueError('Cannot initialize scribble cache without drawing area window')
 
         scale = window.get_scale_factor()
         ww, wh = self.c_da.get_allocated_width() * scale, self.c_da.get_allocated_height() * scale
         try:
             self.scribble_cache = window.create_similar_image_surface(cairo.Format.ARGB32, ww, wh, scale)
-        except ValueError:
-            logger.exception('Error creating highlight cache')
+        except ValueError as e:
+            if e.args == ('invalid enum value: 5',):
+                # Just try again as this error seems to always happen at the first call but not later on
+                # This is the gi introspection of cairo not accepting the value Format.RGB30 (???)
+                try:
+                    self.scribble_cache = window.create_similar_image_surface(cairo.Format.ARGB32, ww, wh, scale)
+                except ValueError:
+                    return
+            else:
+                logger.exception('Error creating highlight cache')
+        except cairo.Error:
+            logger.warning('Failed creating an ARGB32 surface sized {}x{} scale {} for highlight cache'
+                           .format(ww, wh, scale), exc_info=True)
+            return
         self.next_render = 0
 
 
     def prerender(self):
-        """ Clear scribbles to cached.
+        """ Commit scribbles to cache so they are faster to draw on the slide
         """
         if self.scribble_cache is None:
             try:
@@ -429,10 +441,9 @@ class Scribbler(builder.Builder):
             self.next_render = 0
             return
 
-        ww, wh = self.scribble_cache.get_width(), self.scribble_cache.get_height()
-
-        monitor = self.c_da.get_display().get_monitor_at_window(self.c_da.get_parent_window()).get_geometry()
-        pen_scale_factor = max(ww / monitor.width, wh / monitor.height)  # or sqrt of product
+        scale = self.c_da.get_window().get_scale_factor()
+        ww, wh = self.scribble_cache.get_width() / scale, self.scribble_cache.get_height() / scale
+        pen_scale_factor = max(ww / 900, wh / 900)  # or sqrt of product
 
         cairo_context = cairo.Context(self.scribble_cache)
         cairo_context.set_line_cap(cairo.LINE_CAP_ROUND)
@@ -487,22 +498,21 @@ class Scribbler(builder.Builder):
             widget (:class:`~Gtk.DrawingArea`): The widget where to draw the scribbles.
             cairo_context (:class:`~cairo.Context`): The canvas on which to render the drawings
         """
+        scale = widget.get_window().get_scale_factor()
         ww, wh = widget.get_allocated_width(), widget.get_allocated_height()
         cw, ch = self.scribble_cache.get_width(), self.scribble_cache.get_height()
 
-        cairo_context.push_group()
-
         cairo_context.save()
 
-        cairo_context.scale(ww / cw, wh / ch)
+        cairo_context.scale(ww * scale / cw, wh * scale / ch)
         cairo_context.set_source_surface(self.scribble_cache)
         cairo_context.paint()
 
         cairo_context.restore()
 
-        monitor = widget.get_display().get_monitor_at_window(widget.get_parent_window()).get_geometry()
-        pen_scale_factor = max(ww / monitor.width, wh / monitor.height)  # or sqrt of product
+        cairo_context.push_group()
 
+        pen_scale_factor = max(ww / 900, wh / 900)  # or sqrt of product
         if self.scribble_drawing:
             cairo_context.set_line_cap(cairo.LINE_CAP_ROUND)
             color, width, points = self.scribble_list[-1]
@@ -719,7 +729,7 @@ class Scribbler(builder.Builder):
         if self.scribbling_mode:
             return False
 
-        self.off_render.remove(self.scribble_overlay)
+        self.scribble_off_render.remove(self.scribble_overlay)
         self.load_layout('highlight')
 
         self.p_central.queue_draw()
@@ -747,7 +757,7 @@ class Scribbler(builder.Builder):
 
         extras.Cursor.set_cursor(self.scribble_p_da, 'default')
         self.load_layout(None)
-        self.off_render.add(self.scribble_overlay)
+        self.scribble_off_render.add(self.scribble_overlay)
 
         self.get_application().lookup_action('highlight').change_state(GLib.Variant.new_boolean(self.scribbling_mode))
         self.pen_action.set_enabled(self.scribbling_mode)
@@ -826,8 +836,7 @@ class Scribbler(builder.Builder):
         scale = wh / icon.get_height()
 
         dw, dh = self.scribble_p_da.get_allocated_width(), self.scribble_p_da.get_allocated_height()
-        monitor = widget.get_display().get_monitor_at_window(widget.get_parent_window()).get_geometry()
-        pen_scale_factor = max(dw / monitor.width, dh / monitor.height)
+        pen_scale_factor = max(dw / 900, dh / 900)  # or sqrt of product
         width *= pen_scale_factor
 
         cairo_context.push_group()
