@@ -47,7 +47,9 @@ gi.require_version('Gtk', '3.0')
 from gi.repository import GObject, Gtk, Gdk, GLib, GdkPixbuf, Gio
 
 
-from pympress import document, surfacecache, util, pointer, scribble, builder, talk_time, dialog, extras, editable_label
+from pympress import (
+    document, surfacecache, util, pointer, scribble, deck, builder, talk_time, dialog, extras, editable_label
+)
 
 
 class UI(builder.Builder):
@@ -79,7 +81,7 @@ class UI(builder.Builder):
     p_das_next = None
     #: `int` the number of next slides currently on display in the “Next slides” pane, initialized to the maximal number
     next_frames_count = 16
-    #: `list` of :class:`~Gtk.AspectFrame` for the current slide copy in the Presenter window.
+    #: `list` of :class:`~Gtk.AspectFrame` for the current slide copy in the Presenter window.
     p_frame_cur = None
     #: :class:`~Gtk.DrawingArea` for the current slide copy in the Presenter window.
     p_da_cur = None
@@ -134,6 +136,8 @@ class UI(builder.Builder):
 
     #: Class :class:`~pympress.scribble.Scribble` managing drawing by the user on top of the current slide.
     scribbler = None
+    #: Class :class:`~pympress.deck.Overview` displaying a view of all slides
+    deck = None
     #: Class :class:`~pympress.extras.Annotations` managing the display of annotations
     annotations = None
     #: Class :class:`~pympress.extras.Media` managing keeping track of and callbacks on media overlays
@@ -257,6 +261,7 @@ class UI(builder.Builder):
 
         self.zoom = extras.Zoom(self)
         self.scribbler = scribble.Scribbler(self.config, self, self.notes_mode)
+        self.deck = deck.Overview(self.config, self, self.notes_mode)
         self.annotations = extras.Annotations(self)
         self.medias = extras.Media(self, self.config)
         self.laser = pointer.Pointer(self.config, self)
@@ -274,6 +279,7 @@ class UI(builder.Builder):
             name: self.get_object(widget_name) for name, widget_name in self.config.placeable_widgets.items()
         }
         self.placeable_widgets['highlight'] = self.scribbler.scribble_overlay
+        self.placeable_widgets['deck'] = self.deck.deck_viewport
 
         # Initialize windows
         self.make_cwin()
@@ -304,7 +310,17 @@ class UI(builder.Builder):
         self.laser.activate_pointermode()
 
         # Setup screens and show all windows
-        self.setup_screens()
+        self.p_win.parse_geometry(self.config.get('presenter', 'geometry'))
+        self.c_win.parse_geometry(self.config.get('content', 'geometry'))
+
+        screen = self.p_win.get_screen()
+        if type(screen).__name__ != 'GdkWaylandScreen':
+            self.setup_screens(screen, 'at startup')
+        else:
+            self.app.lookup_action('swap-screens').set_enabled(False)
+
+        screen.connect('monitors-changed', self.screens_changed)
+
         self.c_win.show_all()
         self.p_win.show_all()
 
@@ -313,6 +329,9 @@ class UI(builder.Builder):
         self.redraw_panes()
         self.do_page_change(unpause=False)
 
+        if type(screen).__name__ == 'GdkWaylandScreen':
+            GLib.timeout_add(100, self.setup_screens, screen, 'delayed startup')
+
 
     def load_icons(self):
         """ Set the icon list for both windows.
@@ -354,8 +373,8 @@ class UI(builder.Builder):
         pane_handles = self.replace_layout(layout, self.p_central, self.placeable_widgets, self.on_pane_event)
         self.pane_handle_pos.update(pane_handles)
 
-        self.p_frames_next = [self.get_object('p_frame_next{}'.format(n)) for n in range(self.next_frames_count)]
-        self.p_das_next = [self.get_object('p_da_next{}'.format(n)) for n in range(self.next_frames_count)]
+        self.p_frames_next = [self.get_object('p_frame_next{}'.format(n)) for n in range(16)]
+        self.p_das_next = [self.get_object('p_da_next{}'.format(n)) for n in range(16)]
         self.reconfigure_next_frames(None, GLib.Variant.new_int64(self.config.getint('presenter', 'next_slide_count')))
 
         slide_type = self.notes_mode.complement()
@@ -366,6 +385,7 @@ class UI(builder.Builder):
         self.cache.add_widget(self.p_da_notes, self.notes_mode, prerender_enabled = bool(self.notes_mode))
         self.cache.add_widget(self.scribbler.scribble_p_da, slide_type, prerender_enabled = False)
         self.cache.add_widget(self.scribbler.scribble_p_da, slide_type, zoomed = True)
+        self.cache.add_widget(self.deck.deck0, slide_type, prerender_enabled = False, ignore_max = True)
 
         # set default value
         self.page_number.set_last(self.doc.pages_number())
@@ -379,7 +399,7 @@ class UI(builder.Builder):
 
 
     def reconfigure_next_frames(self, gaction, param):
-        """ Set the number of next frames to preview the the “next slides” panel
+        """ Callback to set the number of next frames to preview the the “next slides” panel
 
         Args:
             gaction (:class:`~Gio.Action`): the action triggering the call
@@ -391,6 +411,18 @@ class UI(builder.Builder):
         if n_frames > 16:
             n_frames = 16
 
+        self.reflow_next_frames(n_frames)
+        self.next_frames_count = n_frames
+        self.app.set_action_state('next-frames', n_frames)
+        self.config.set('presenter', 'next_slide_count', str(n_frames))
+
+
+    def reflow_next_frames(self, n_frames=None):
+        """ Set the number of next frames to preview the the “next slides” panel
+
+        Args:
+            n_frames (`int`): the number of frames
+        """
         for n in range(self.next_frames_count):
             self.grid_next.remove(self.p_frames_next[n])
 
@@ -398,34 +430,86 @@ class UI(builder.Builder):
             self.grid_next.remove_row(n)
             self.grid_next.remove_column(n)
 
-        rows = int(round(math.sqrt(n_frames)))
-        cols = (n_frames + rows - 1) // rows
+        if n_frames is None:
+            n_frames = self.next_frames_count
+
+        ww, wh = self.grid_next.get_allocated_width(), self.grid_next.get_allocated_height()
+        rows, cols = self.compute_frame_grid(ww / wh, n_frames)
 
         for n in range(n_frames):
             self.grid_next.attach(self.p_frames_next[n], n % cols, n // cols, 1, 1)
 
-        self.next_frames_count = n_frames
-        self.app.set_action_state('next-frames', n_frames)
-        self.config.set('presenter', 'next_slide_count', str(n_frames))
 
+    def compute_frame_grid(self, grid_ar, n_frames):
+        """ Determine the arragement of frames in a grid to maximise their size given respective aspect ratios
 
-    def setup_screens(self):
-        """ Sets up the position of the windows.
+        Args:
+            grid_ar (`float`): aspect ratio of grid containing the slides
+            n_frames (`int`): the number of frames
         """
-        self.p_win.parse_geometry(self.config.get('presenter', 'geometry'))
-        self.c_win.parse_geometry(self.config.get('content', 'geometry'))
+        if not n_frames:
+            return 0, 0
 
-        c_full = self.app.get_action_state('content-fullscreen')
-        p_full = self.app.get_action_state('presenter-fullscreen')
+        page_ratio = self.doc.page(self.preview_page).get_aspect_ratio(self.notes_mode.complement())
+        grid_ar /= page_ratio
 
-        if not c_full and not p_full:
-            # Just restored window sizes, that’s enough
+        # works best when n_frames = rows * cols * grid_ar
+        rows = math.sqrt(n_frames / grid_ar)
+
+        # If rows < 1 always ceil
+        if rows < 1:
+            return 1, n_frames
+
+        # Check which of ceiling or flooring rows is better
+        rows = math.floor(rows)
+        f_cols, c_cols = (n_frames + rows - 1) // rows, (n_frames + rows) // (rows + 1)
+
+        if min(f_cols, grid_ar * rows) > min(c_cols, grid_ar * (rows + 1)):
+            return rows, f_cols
+        else:
+            return rows + 1, c_cols
+
+
+    def screens_changed(self, screen):
+        """ Handle ::monitors-changed events
+
+        Args:
+            screen (:class:`~Gdk.Screen`):  the screen
+        """
+        self.setup_screens(screen, 'at monitors-changed event')
+        GLib.timeout_add(1000, self.setup_screens, screen, 'delayed after monitors-changed event')
+
+
+    def setup_screens(self, screen, event_name=''):
+        """ If multiple monitors, fullscreen windows on monitors according to config.
+
+        Args:
+            screen (:class:`~Gdk.Screen`):  the screen
+            event_name (`str`):  a description of what caused the screen setup event, for debugging
+        """
+        display = Gdk.Display.get_default()
+        try:
+            c_monitor, p_monitor, primary, non_primary = util.Monitor.lookup_monitors(display, self.c_win, self.p_win)
+        except util.NoMonitorPositions:
             return
 
-        # If multiple monitors, apply windows to monitors according to config
-        screen = self.p_win.get_screen()
+        # Noisy, but just for debug level
+        logger.debug('Detecting monitor layout ' + event_name + ': ' + ', '.join([
+            'Content window is at monitor ' + c_monitor.name,
+            'Presenter window is at monitor ' + p_monitor.name,
+            'Primary monitor is (or overlapped with) ' + primary.name,
+            'No non-primary monitor available' if primary.equal(non_primary)
+            else 'A non-primary monitor is (or overlapped with) ' + non_primary.name
+        ]))
 
-        if screen.get_n_monitors() <= 1:
+        c_full = self.config.getboolean('content', 'start_fullscreen')
+        p_full = self.config.getboolean('presenter', 'start_fullscreen')
+        wayland = type(display).__name__ == 'GdkWaylandDisplay'
+
+        if not c_full and not p_full or c_monitor is None or p_monitor is None:
+            return
+
+        if primary.equal(non_primary):
             logger.warning(_('Not starting content or presenter window full screen ' +
                              'because there is only one monitor'))
 
@@ -433,47 +517,59 @@ class UI(builder.Builder):
 
         else:
             # To start fullscreen, we need to ensure windows are on individual monitors
-            c_monitor = screen.get_monitor_at_point(*self.c_win.get_position())
-            p_monitor = screen.get_monitor_at_point(*self.p_win.get_position())
-            primary = screen.get_primary_monitor()
-            if c_monitor == p_monitor:
-                warning = _('Content and presenter window must not be on the same monitor if you start full screen!')
-                logger.warning(warning)
-
-                if p_monitor == primary:
-                    # move content somewhere else
-                    self.move_window(screen, self.c_win, c_monitor, (primary + 1) % screen.get_n_monitors())
-                else:
-                    # move presenter to primary
-                    self.move_window(screen, self.p_win, p_monitor, primary)
+            if not c_monitor.equal(p_monitor):
+                pass
+            elif wayland:
+                # On Wayland, we can’t move windows, and they happen to be on the same monitor.
+                # So if we are to fullscreen one, just do it on another monitor
+                if c_full:
+                    c_monitor = p_monitor.least_intersection([primary, non_primary])
+                elif p_full:
+                    p_monitor = c_monitor.least_intersection([primary, non_primary])
+            elif c_monitor.equal(primary):
+                self.move_window(self.c_win, c_monitor, non_primary)
+                c_monitor = non_primary
+            else:  # not p_monitor.equal(primary)
+                self.move_window(self.p_win, p_monitor, primary)
+                p_monitor = primary
 
         if p_full:
-            self.p_win.fullscreen()
+            self.p_win.fullscreen_on_monitor(self.p_win.get_screen(), p_monitor.monitor_number)
+        elif not wayland:
+            self.p_win.unfullscreen()
 
         if c_full:
-            self.c_win.fullscreen()
+            self.c_win.fullscreen_on_monitor(self.c_win.get_screen(), c_monitor.monitor_number)
             self.set_screensaver(True)
+        else:
+            # Let Wayland do its own un-fullscreening as we can’t move the windows afterwards and messing with it
+            # results in a non-fullscreen window that is out of bounds of the remaining monitors.
+            if not wayland:
+                self.c_win.unfullscreen()
+            self.set_screensaver(False)
 
         self.app.set_action_state('content-fullscreen', c_full)
         self.app.set_action_state('presenter-fullscreen', p_full)
 
 
-    def move_window(self, screen, win, from_monitor, to_monitor):
+    def move_window(self, win, from_bounds, to_bounds):
         """ Move window from monitor number from_monitor to monitor to_monitor.
         """
         x, y, w, h = win.get_position() + win.get_size()
         win_state = win.get_window().get_state() if win.get_window() is not None else 0
 
+        logger.debug('Moving {} window from screen {} to screen {}'.format(
+            'Content' if win is self.c_win else 'Presenter', from_bounds.name, to_bounds.name
+        ))
+
         if (win_state & Gdk.WindowState.FULLSCREEN) != 0:
             win.unfullscreen()
         if (win_state & Gdk.WindowState.MAXIMIZED) != 0:
             win.unmaximize()
 
-        to_bounds = screen.get_monitor_geometry(to_monitor)
         to_w = min(w, to_bounds.width)
         to_h = min(h, to_bounds.height)
 
-        from_bounds = screen.get_monitor_geometry(from_monitor)
         # Get fraction of free space that is left or top of window
         x = (max(0, x - from_bounds.x) / (from_bounds.width - w)) if w < from_bounds.width else 0
         y = (max(0, y - from_bounds.y) / (from_bounds.height - h)) if h < from_bounds.height else 0
@@ -542,6 +638,8 @@ class UI(builder.Builder):
         elif widget is self.p_da_cur:
             self.medias.resize('presenter')
 
+        widget.queue_draw()
+
 
     def on_configure_win(self, widget, event):
         """ Manage "configure" events for both window widgets.
@@ -551,16 +649,23 @@ class UI(builder.Builder):
             event (:class:`~Gdk.Event`):  the GTK event, which contains the new dimensions of the widget
         """
         geom = '{}x{}{:+}{:+}'.format(*widget.get_size(), *widget.get_position())
+        win_state = widget.get_window().get_state() if widget.get_window() is not None else 0
+        win_state &= Gdk.WindowState.FULLSCREEN | Gdk.WindowState.MAXIMIZED
 
         if widget is self.p_win:
-            self.config.set('presenter', 'geometry', geom)
+            if win_state == 0:
+                self.config.set('presenter', 'geometry', geom)
             cw = self.p_central.get_allocated_width()
             ch = self.p_central.get_allocated_height()
-            self.scribbler.off_render.set_size_request(cw, ch)
+            self.scribbler.scribble_off_render.set_size_request(cw, ch)
+            self.deck.deck_off_render.set_size_request(cw, ch)
+
+            if self.deck.deck_mode:
+                self.deck.reset_grid()
 
             self.adjust_bottom_bar_font()
 
-        elif widget is self.c_win:
+        elif widget is self.c_win and win_state == 0:
             self.config.set('content', 'geometry', geom)
 
 
@@ -580,6 +685,7 @@ class UI(builder.Builder):
         This is very useful on windows where resizing gets sluggish if we try to redraw while resizing.
         """
         self.resize_panes = False
+        self.reflow_next_frames()
         self.p_da_cur.queue_draw()
         for da in self.p_das_next:
             da.queue_draw()
@@ -725,6 +831,7 @@ class UI(builder.Builder):
             self.talk_time.pause()
             self.talk_time.reset_timer()
             self.page_number.setup_doc_callbacks(self.doc)
+            self.deck.setup_doc_callbacks(self.doc)
 
         self.do_page_change(unpause=False)
 
@@ -844,16 +951,16 @@ class UI(builder.Builder):
         dialog.set_default_response(Gtk.ResponseType.OK)
         dialog.set_position(Gtk.WindowPosition.CENTER)
 
-        filter = Gtk.FileFilter()
-        filter.set_name(_('PDF files'))
-        filter.add_mime_type('application/pdf')
-        filter.add_pattern('*.pdf')
-        dialog.add_filter(filter)
+        file_filter = Gtk.FileFilter()
+        file_filter.set_name(_('PDF files'))
+        file_filter.add_mime_type('application/pdf')
+        file_filter.add_pattern('*.pdf')
+        dialog.add_filter(file_filter)
 
-        filter = Gtk.FileFilter()
-        filter.set_name(_('All files'))
-        filter.add_pattern('*')
-        dialog.add_filter(filter)
+        file_filter = Gtk.FileFilter()
+        file_filter.set_name(_('All files'))
+        file_filter.add_pattern('*')
+        dialog.add_filter(file_filter)
 
         response = dialog.run()
 
@@ -873,16 +980,16 @@ class UI(builder.Builder):
         dialog.set_default_response(Gtk.ResponseType.OK)
         dialog.set_position(Gtk.WindowPosition.CENTER)
 
-        filter = Gtk.FileFilter()
-        filter.set_name(_('PDF files'))
-        filter.add_mime_type('application/pdf')
-        filter.add_pattern('*.pdf')
-        dialog.add_filter(filter)
+        file_filter = Gtk.FileFilter()
+        file_filter.set_name(_('PDF files'))
+        file_filter.add_mime_type('application/pdf')
+        file_filter.add_pattern('*.pdf')
+        dialog.add_filter(file_filter)
 
-        filter = Gtk.FileFilter()
-        filter.set_name(_('All files'))
-        filter.add_pattern('*')
-        dialog.add_filter(filter)
+        file_filter = Gtk.FileFilter()
+        file_filter.set_name(_('All files'))
+        file_filter.add_pattern('*')
+        dialog.add_filter(file_filter)
 
         response = dialog.run()
 
@@ -1209,13 +1316,24 @@ class UI(builder.Builder):
                 return
 
             # Cache miss: render the page, and save it to the cache
-            pb = window.create_similar_image_surface(cairo.Format.RGB24, ww * scale, wh * scale, scale)
+            try:
+                pb = window.create_similar_image_surface(cairo.Format.RGB24, ww * scale, wh * scale, scale)
+            except cairo.Error:
+                logger.warning('Failed creating an RGB24 surface sized {}x{} scale {} for widget {}'
+                               .format(ww * scale, wh * scale, scale, name), exc_info=True)
+                if widget is self.c_da:
+                    cairo_context.save()
+                    cairo_context.set_source_rgb(0, 0, 0)
+                    cairo_context.fill()
+                    cairo_context.paint()
+                    cairo_context.restore()
+                return
 
             cairo_prerender = cairo.Context(pb)
             cairo_prerender.transform(zoom_matrix)
             page.render_cairo(cairo_prerender, ww, wh, wtype)
 
-            self.cache.set(name, nb, pb)
+            self.cache.put(name, nb, pb)
 
             cairo_context.set_source_surface(pb, 0, 0)
             cairo_context.paint()
@@ -1303,6 +1421,8 @@ class UI(builder.Builder):
             return True
         elif self.scribbler.try_cancel():
             return True
+        elif self.deck.try_cancel():
+            return True
         elif self.annotations.try_cancel():
             return True
         elif self.autoplay.stop_looping():
@@ -1481,9 +1601,7 @@ class UI(builder.Builder):
         window = widget.get_window()
         cur_state = (window.get_state() & Gdk.WindowState.FULLSCREEN) if window is not None else False
 
-        if cur_state == toggle_to:
-            return False
-        elif cur_state:
+        if cur_state:
             widget.unfullscreen()
         else:
             widget.fullscreen()
@@ -1570,17 +1688,18 @@ class UI(builder.Builder):
     def swap_screens(self, *args):
         """ Swap the monitors on which each window is displayed (if there are 2 monitors at least).
         """
-        screen = self.p_win.get_screen()
-
-        # Though Gtk.Window is a Gtk.Widget get_parent_window() actually returns None on self.{c,p}_win
-        p_monitor = screen.get_monitor_at_point(*self.p_win.get_position())
-        c_monitor = screen.get_monitor_at_point(*self.c_win.get_position())
-
-        if screen.get_n_monitors() == 1 or p_monitor == c_monitor:
+        display = Gdk.Display.get_default()
+        try:
+            c_monitor, p_monitor, primary, non_primary = util.Monitor.lookup_monitors(display, self.c_win, self.p_win)
+        except util.NoMonitorPositions:
             return
 
-        self.move_window(screen, self.c_win, c_monitor, p_monitor)
-        self.move_window(screen, self.p_win, p_monitor, c_monitor)
+        if primary.equal(non_primary) == 1 or p_monitor.equal(c_monitor):
+            return
+
+        logger.debug('Swapping screens:')
+        self.move_window(self.c_win, c_monitor, p_monitor)
+        self.move_window(self.p_win, p_monitor, c_monitor)
 
 
     def switch_blanked(self, gaction, param):
@@ -1611,6 +1730,8 @@ class UI(builder.Builder):
         """
         if self.scribbler.scribbling_mode:
             return 'highlight'
+        elif self.deck.deck_mode:
+            return 'deck'
         elif notes_mode.direction() == 'page number':
             return 'note_pages'
         elif notes_mode:
@@ -1754,6 +1875,8 @@ class UI(builder.Builder):
                     size = parent.get_allocated_height()
                 parent.set_position(self.pane_handle_pos[parent] * size)
 
+        GLib.idle_add(self.redraw_panes)
+
         self.annotations.load_annotations(self.doc.page(self.preview_page))
         gaction.change_state(GLib.Variant.new_boolean(self.show_annotations))
 
